from .utils.validator import Validator
from .utils.base_service import BaseService
from ..net.transport.serializer import Serializer
from ..models.utils.cast_models import cast_models
from ..models import (
    CheckoutSubscriptionsBody,
    InlineResponse200_20,
    InlineResponse200_5,
    InlineResponse200_6,
    InlineResponse200_7,
    InlineResponse200_8,
    InlineResponse200_9,
    PaymentsSubscriptionsBody,
    SubscriptionsSubscriptionIdBody,
)


class SubscriptionService(BaseService):

    @cast_models
    def get_subscription_list(
        self,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        billing: str = None,
        customer: str = None,
        status: str = None,
        product: str = None,
        starting_after: str = None,
        ending_before: str = None,
        limit: str = None,
        idempotency: str = None,
    ) -> InlineResponse200_5:
        """Retrieve a list of subscriptions. You can filter the list with query parameters.

        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param billing: Method of billing. One of the following, pay_automatically, send_invoice., defaults to None
        :type billing: str, optional
        :param customer: ID of the customer. String starting with cus_, defaults to None
        :type customer: str, optional
        :param status: Status of the subscription. One of the following, active, canceled, trialing, defaults to None
        :type status: str, optional
        :param product: ID of a 'product' object. The product must have type set to service. String starting with product_. Filter for one product at a time., defaults to None
        :type product: str, optional
        :param starting_after: The ID of a record in the list. The list begins with the record that was created next after the record with this ID. Use this filter to get the next page of results. Relevant when ending_before is not used., defaults to None
        :type starting_after: str, optional
        :param ending_before: The ID of a record in the list. The list ends with the last record that was created before the record with this ID. Use this filter to get the previous page of results., defaults to None
        :type ending_before: str, optional
        :param limit: The maximum number of subscriptions to return. Range, 1-100. Default is 10., defaults to None
        :type limit: str, optional
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: List of subscriptions,
        :rtype: InlineResponse200_5
        """

        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(billing)
        Validator(str).is_optional().validate(customer)
        Validator(str).is_optional().validate(status)
        Validator(str).is_optional().validate(product)
        Validator(str).is_optional().validate(starting_after)
        Validator(str).is_optional().validate(ending_before)
        Validator(str).is_optional().validate(limit)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/payments/subscriptions", self.get_default_headers()
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .add_query("billing", billing, explode=False)
            .add_query("customer", customer, explode=False)
            .add_query("status", status, explode=False)
            .add_query("product", product, explode=False)
            .add_query("starting_after", starting_after, explode=False)
            .add_query("ending_before", ending_before, explode=False)
            .add_query("limit", limit, explode=False)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_5._unmap(response)

    @cast_models
    def create_subscription(
        self,
        request_body: PaymentsSubscriptionsBody,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        idempotency: str = None,
    ) -> InlineResponse200_6:
        """Create a subscription for regular, automatic payments.

        :param request_body: The request body.
        :type request_body: PaymentsSubscriptionsBody
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Create a subscription
        :rtype: InlineResponse200_6
        """

        Validator(PaymentsSubscriptionsBody).validate(request_body)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/payments/subscriptions", self.get_default_headers()
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_6._unmap(response)

    @cast_models
    def get_subscription(
        self,
        subscription_id: str,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        idempotency: str = None,
    ) -> InlineResponse200_6:
        """Retrieve the details of a subscription.

        :param subscription_id: ID of the subscription. String starting with sub_.
        :type subscription_id: str
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Get subscription details
        :rtype: InlineResponse200_6
        """

        Validator(str).validate(subscription_id)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/payments/subscriptions/{{subscriptionId}}",
                self.get_default_headers(),
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .add_path("subscriptionId", subscription_id)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_6._unmap(response)

    @cast_models
    def update_subscription(
        self,
        request_body: SubscriptionsSubscriptionIdBody,
        subscription_id: str,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        idempotency: str = None,
    ) -> InlineResponse200_6:
        """Update the details of a subscription.

        :param request_body: The request body.
        :type request_body: SubscriptionsSubscriptionIdBody
        :param subscription_id: ID of the subscription. String starting with **sub_**.
        :type subscription_id: str
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The subscription after updated
        :rtype: InlineResponse200_6
        """

        Validator(SubscriptionsSubscriptionIdBody).validate(request_body)
        Validator(str).validate(subscription_id)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/payments/subscriptions/{{subscriptionId}}",
                self.get_default_headers(),
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .add_path("subscriptionId", subscription_id)
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_6._unmap(response)

    @cast_models
    def cancel_subscription(
        self,
        subscription_id: str,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        idempotency: str = None,
    ) -> InlineResponse200_6:
        """Cancel a subscription.

        :param subscription_id: ID of the subscription. String starting with **sub_**.
        :type subscription_id: str
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The canceled subscription
        :rtype: InlineResponse200_6
        """

        Validator(str).validate(subscription_id)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/payments/subscriptions/{{subscriptionId}}",
                self.get_default_headers(),
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .add_path("subscriptionId", subscription_id)
            .serialize()
            .set_method("DELETE")
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_6._unmap(response)

    @cast_models
    def delete_subscription_discount(
        self,
        subscription_id: str,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        idempotency: str = None,
    ) -> InlineResponse200_7:
        """Delete the discount that was assigned to a subscription. This method does not affect the coupon that the discount was derived from.

        :param subscription_id: ID of the subscription. String starting with **sub_**.
        :type subscription_id: str
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: discount was deleted
        :rtype: InlineResponse200_7
        """

        Validator(str).validate(subscription_id)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/payments/subscriptions/{{subscriptionId}}/discount",
                self.get_default_headers(),
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .add_path("subscriptionId", subscription_id)
            .serialize()
            .set_method("DELETE")
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_7._unmap(response)

    @cast_models
    def create_subscription_by_hosted_page(
        self,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        request_body: CheckoutSubscriptionsBody = None,
        idempotency: str = None,
    ) -> InlineResponse200_8:
        """Create a subscription using a hosted page. Relevant to card payments.

        :param request_body: The request body., defaults to None
        :type request_body: CheckoutSubscriptionsBody, optional
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Subscription hosted Page was created.
        :rtype: InlineResponse200_8
        """

        Validator(CheckoutSubscriptionsBody).is_optional().validate(request_body)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/checkout/subscriptions", self.get_default_headers()
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_8._unmap(response)

    @cast_models
    def complete_subscription_cycle(self, subscription_id: str) -> InlineResponse200_9:
        """Cancel the subscription and create an invoice. This method is for testing purposes and runs only in the sandbox.

        :param subscription_id: ID of the subscription. String starting with sub_.
        :type subscription_id: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The subscription
        :rtype: InlineResponse200_9
        """

        Validator(str).validate(subscription_id)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/payments/subscriptions/{{subscriptionId}}/complete_cycle",
                self.get_default_headers(),
            )
            .add_path("subscriptionId", subscription_id)
            .serialize()
            .set_method("POST")
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_9._unmap(response)

    @cast_models
    def simulate_start_new_cycle(self, subscription_id: str) -> InlineResponse200_9:
        """End a subscription cycle, create an invoice and move the subscription to the next cycle. This method is for testing purposes and runs only in the sandbox.

        :param subscription_id: ID of the subscription. String starting with sub_.
        :type subscription_id: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Subscription details
        :rtype: InlineResponse200_9
        """

        Validator(str).validate(subscription_id)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/payments/subscriptions/{{subscriptionId}}/start_new_cycle",
                self.get_default_headers(),
            )
            .add_path("subscriptionId", subscription_id)
            .serialize()
            .set_method("POST")
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_9._unmap(response)

    @cast_models
    def get_subscription_discount_by_id(
        self,
        discount_id: str,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        idempotency: str = None,
    ) -> InlineResponse200_20:
        """Retrieve a discount for a subscription.

        :param discount_id: discount Id
        :type discount_id: str
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Get discount details of a subscription by Discount Id.
        :rtype: InlineResponse200_20
        """

        Validator(str).validate(discount_id)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/subscriptions/discount/{{discountId}}",
                self.get_default_headers(),
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .add_path("discountId", discount_id)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_20._unmap(response)



from .utils.json_map import JsonMap
from .utils.base_model import BaseModel


@JsonMap(
    {
        "content_length": "content-length",
        "content_type": "content-type",
        "date_": "date",
    }
)
class ListWebhooksResponseAttemptsHttpResponseHeaders(BaseModel):
    """Headers of the HTTP response from the configured webhook destination, with the value received.

    :param connection: Control options for the connection., defaults to None
    :type connection: str, optional
    :param content_length: Length of the content in bytes.ook destination., defaults to None
    :type content_length: float, optional
    :param content_type: MIME type of the content., defaults to None
    :type content_type: str, optional
    :param date_: Timestamp of the response in HTTP-date format (RFC 9110)., defaults to None
    :type date_: str, optional
    :param server: Name of the server., defaults to None
    :type server: str, optional
    """

    def __init__(
        self,
        connection: str = None,
        content_length: float = None,
        content_type: str = None,
        date_: str = None,
        server: str = None,
    ):
        """Headers of the HTTP response from the configured webhook destination, with the value received.

        :param connection: Control options for the connection., defaults to None
        :type connection: str, optional
        :param content_length: Length of the content in bytes.ook destination., defaults to None
        :type content_length: float, optional
        :param content_type: MIME type of the content., defaults to None
        :type content_type: str, optional
        :param date_: Timestamp of the response in HTTP-date format (RFC 9110)., defaults to None
        :type date_: str, optional
        :param server: Name of the server., defaults to None
        :type server: str, optional
        """
        self.connection = self._define_str("connection", connection, nullable=True)
        self.content_length = self._define_number(
            "content_length", content_length, nullable=True
        )
        self.content_type = self._define_str(
            "content_type", content_type, nullable=True
        )
        self.date_ = self._define_str("date_", date_, nullable=True)
        self.server = self._define_str("server", server, nullable=True)



from enum import Enum


class NextAction(Enum):
    """An enumeration representing different categories.

    :cvar _3DVERIFICATION: "3d_verification"
    :vartype _3DVERIFICATION: str
    :cvar PENDINGCAPTURE: "pending_capture"
    :vartype PENDINGCAPTURE: str
    :cvar PENDINGCONFIRMATION: "pending_confirmation"
    :vartype PENDINGCONFIRMATION: str
    :cvar NOTAPPLICABLE: "not_applicable"
    :vartype NOTAPPLICABLE: str
    """

    _3DVERIFICATION = "3d_verification"
    PENDINGCAPTURE = "pending_capture"
    PENDINGCONFIRMATION = "pending_confirmation"
    NOTAPPLICABLE = "not_applicable"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(map(lambda x: x.value, NextAction._member_map_.values()))



from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .hosted_beneficiary_token_response import HostedBeneficiaryTokenResponse
from .status import Status


@JsonMap({})
class InlineResponse200_60(BaseModel):
    """InlineResponse200_60

    :param data: data, defaults to None
    :type data: HostedBeneficiaryTokenResponse, optional
    :param status: status, defaults to None
    :type status: Status, optional
    """

    def __init__(
        self, data: HostedBeneficiaryTokenResponse = None, status: Status = None
    ):
        """InlineResponse200_60

        :param data: data, defaults to None
        :type data: HostedBeneficiaryTokenResponse, optional
        :param status: status, defaults to None
        :type status: Status, optional
        """
        self.data = self._define_object(data, HostedBeneficiaryTokenResponse)
        self.status = self._define_object(status, Status)



from __future__ import annotations
from typing import List
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .refund_ewallets import RefundEwallets


@JsonMap({"id_": "id"})
class Refund(BaseModel):
    """Refund

    :param amount: Amount of the refund, in units defined by currency in the original payment. Decimal, including the correct number of decimal places for the currency exponent, as defined in ISO 2417:2015., defaults to None
    :type amount: float, optional
    :param created_at: Time of creation of this refund, in Unix time. Response only., defaults to None
    :type created_at: float, optional
    :param currency: In transactions without FX, defines the currency of the transaction. Three-letter ISO 4217 code.\<BR\> In FX transactions:\<BR\> * When `fixed_side` is **buy**, it is the currency received in the Rapyd wallet.\<BR\> * When `fixed_side` is **sell**, it is the currency charged to the buyer., defaults to None
    :type currency: str, optional
    :param ewallets: An array of one object, which contains the following fields: * ewallet_id - The ID of the wallet that the money is transferred from, a string starting with ewallet_. * amount - Amount of the transaction in currency units defined in currency. * percent - A decimal number representing a percentage of the total payout. Response only., defaults to None
    :type ewallets: List[RefundEwallets], optional
    :param failure_reason: Indicates the reason that the refund failed., defaults to None
    :type failure_reason: str, optional
    :param fixed_side: Indicates whether the FX rate is fixed for the buy side or for the sell side. Relevant for refunds with foreign exchange. Response only., defaults to None
    :type fixed_side: str, optional
    :param fx_rate: Exchange rate for the transaction. When fixed_side is buy, fx_rate is the buy rate. When fixed_side is sell, fx_rate is the sell rate. Decimal number as string. Relevant to refunds with foreign exchange. Response only., defaults to None
    :type fx_rate: str, optional
    :param id_: ID of the Refund object. String starting with **refund_**., defaults to None
    :type id_: str, optional
    :param merchant_debited_amount: Amount debited from the merchant. Relevant to refunds with foreign exchange. Response only., defaults to None
    :type merchant_debited_amount: str, optional
    :param merchant_debited_currency: Indicates the currency that is debited from the merchant. Three-letter ISO 4217 code. Relevant to refunds with foreign exchange. Response only., defaults to None
    :type merchant_debited_currency: str, optional
    :param merchant_reference_id: Merchant-defined ID. Limited to 255 characters., defaults to None
    :type merchant_reference_id: str, optional
    :param metadata: A JSON object defined by the client., defaults to None
    :type metadata: dict, optional
    :param payment: D of the Payment object that the refund is credited against. String starting with payment_., defaults to None
    :type payment: str, optional
    :param payment_created_at: Time that the original payment was created, in Unix time. Response only., defaults to None
    :type payment_created_at: float, optional
    :param payment_method_type: The original payment payment method type. Use List Payment Methods by Country for a list of supported types for a country, defaults to None
    :type payment_method_type: str, optional
    :param proportional_refund: Indicates whether the refund was returned in proportion to the amounts received by the wallets in the payment. Relevant to a refund for a payment split among multiple wallets., defaults to None
    :type proportional_refund: bool, optional
    :param reason: Description of the reason for the refund, provided by the merchant., defaults to None
    :type reason: str, optional
    :param receipt_number: Number of the receipt for the refund, provided by the merchant. Response only., defaults to None
    :type receipt_number: float, optional
    :param status: Indicates the status of the refund operation, defaults to None
    :type status: str, optional
    :param updated_at: Time that this refund was last updated, in Unix time. Response only., defaults to None
    :type updated_at: float, optional
    """

    def __init__(
        self,
        amount: float = None,
        created_at: float = None,
        currency: str = None,
        ewallets: List[RefundEwallets] = None,
        failure_reason: str = None,
        fixed_side: str = None,
        fx_rate: str = None,
        id_: str = None,
        merchant_debited_amount: str = None,
        merchant_debited_currency: str = None,
        merchant_reference_id: str = None,
        metadata: dict = None,
        payment: str = None,
        payment_created_at: float = None,
        payment_method_type: str = None,
        proportional_refund: bool = None,
        reason: str = None,
        receipt_number: float = None,
        status: str = None,
        updated_at: float = None,
    ):
        """Refund

        :param amount: Amount of the refund, in units defined by currency in the original payment. Decimal, including the correct number of decimal places for the currency exponent, as defined in ISO 2417:2015., defaults to None
        :type amount: float, optional
        :param created_at: Time of creation of this refund, in Unix time. Response only., defaults to None
        :type created_at: float, optional
        :param currency: In transactions without FX, defines the currency of the transaction. Three-letter ISO 4217 code.\<BR\> In FX transactions:\<BR\> * When `fixed_side` is **buy**, it is the currency received in the Rapyd wallet.\<BR\> * When `fixed_side` is **sell**, it is the currency charged to the buyer., defaults to None
        :type currency: str, optional
        :param ewallets: An array of one object, which contains the following fields: * ewallet_id - The ID of the wallet that the money is transferred from, a string starting with ewallet_. * amount - Amount of the transaction in currency units defined in currency. * percent - A decimal number representing a percentage of the total payout. Response only., defaults to None
        :type ewallets: List[RefundEwallets], optional
        :param failure_reason: Indicates the reason that the refund failed., defaults to None
        :type failure_reason: str, optional
        :param fixed_side: Indicates whether the FX rate is fixed for the buy side or for the sell side. Relevant for refunds with foreign exchange. Response only., defaults to None
        :type fixed_side: str, optional
        :param fx_rate: Exchange rate for the transaction. When fixed_side is buy, fx_rate is the buy rate. When fixed_side is sell, fx_rate is the sell rate. Decimal number as string. Relevant to refunds with foreign exchange. Response only., defaults to None
        :type fx_rate: str, optional
        :param id_: ID of the Refund object. String starting with **refund_**., defaults to None
        :type id_: str, optional
        :param merchant_debited_amount: Amount debited from the merchant. Relevant to refunds with foreign exchange. Response only., defaults to None
        :type merchant_debited_amount: str, optional
        :param merchant_debited_currency: Indicates the currency that is debited from the merchant. Three-letter ISO 4217 code. Relevant to refunds with foreign exchange. Response only., defaults to None
        :type merchant_debited_currency: str, optional
        :param merchant_reference_id: Merchant-defined ID. Limited to 255 characters., defaults to None
        :type merchant_reference_id: str, optional
        :param metadata: A JSON object defined by the client., defaults to None
        :type metadata: dict, optional
        :param payment: D of the Payment object that the refund is credited against. String starting with payment_., defaults to None
        :type payment: str, optional
        :param payment_created_at: Time that the original payment was created, in Unix time. Response only., defaults to None
        :type payment_created_at: float, optional
        :param payment_method_type: The original payment payment method type. Use List Payment Methods by Country for a list of supported types for a country, defaults to None
        :type payment_method_type: str, optional
        :param proportional_refund: Indicates whether the refund was returned in proportion to the amounts received by the wallets in the payment. Relevant to a refund for a payment split among multiple wallets., defaults to None
        :type proportional_refund: bool, optional
        :param reason: Description of the reason for the refund, provided by the merchant., defaults to None
        :type reason: str, optional
        :param receipt_number: Number of the receipt for the refund, provided by the merchant. Response only., defaults to None
        :type receipt_number: float, optional
        :param status: Indicates the status of the refund operation, defaults to None
        :type status: str, optional
        :param updated_at: Time that this refund was last updated, in Unix time. Response only., defaults to None
        :type updated_at: float, optional
        """
        self.amount = self._define_number("amount", amount, nullable=True)
        self.created_at = self._define_number("created_at", created_at, nullable=True)
        self.currency = self._define_str(
            "currency",
            currency,
            nullable=True,
            pattern="/^AED|AFN|ALL|AMD|ANG|AOA|ARS|AUD|AWG|AZN|BAM|BBD|BDT|BGN|BHD|BIF|BMD|BND|BOB|BRL|BSD|BTN|BWP|BYR|BZD|CAD|CDF|CHF|CLP|CNY|COP|CRC|CUC|CUP|CVE|CZK|DJF|DKK|DOP|DZD|EGP|ERN|ETB|EUR|FJD|FKP|GBP|GEL|GGP|GHS|GIP|GMD|GNF|GTQ|GYD|HKD|HNL|HRK|HTG|HUF|IDR|ILS|IMP|INR|IQD|IRR|ISK|JEP|JMD|JOD|JPY|KES|KGS|KHR|KMF|KPW|KRW|KWD|KYD|KZT|LAK|LBP|LKR|LRD|LSL|LYD|MAD|MDL|MGA|MKD|MMK|MNT|MOP|MRO|MUR|MVR|MWK|MXN|MYR|MZN|NAD|NGN|NIO|NOK|NPR|NZD|OMR|PAB|PEN|PGK|PHP|PKR|PLN|PYG|QAR|RON|RSD|RUB|RWF|SAR|SBD|SCR|SDG|SEK|SGD|SHP|SLL|SOS|SPL|SRD|STD|SVC|SYP|SZL|THB|TJS|TMT|TND|TOP|TRY|TTD|TVD|TWD|TZS|UAH|UGX|USD|UYU|UZS|VEF|VND|VUV|WST|XAF|XCD|XDR|XOF|XPF|YER|ZAR|ZMW|ZWD$/",
        )
        self.ewallets = self._define_list(ewallets, RefundEwallets)
        self.failure_reason = self._define_str(
            "failure_reason", failure_reason, nullable=True
        )
        self.fixed_side = self._define_str("fixed_side", fixed_side, nullable=True)
        self.fx_rate = self._define_str("fx_rate", fx_rate, nullable=True)
        self.id_ = self._define_str("id_", id_, nullable=True)
        self.merchant_debited_amount = self._define_str(
            "merchant_debited_amount", merchant_debited_amount, nullable=True
        )
        self.merchant_debited_currency = self._define_str(
            "merchant_debited_currency", merchant_debited_currency, nullable=True
        )
        self.merchant_reference_id = self._define_str(
            "merchant_reference_id", merchant_reference_id, nullable=True
        )
        self.metadata = metadata
        self.payment = self._define_str("payment", payment, nullable=True)
        self.payment_created_at = self._define_number(
            "payment_created_at", payment_created_at, nullable=True
        )
        self.payment_method_type = self._define_str(
            "payment_method_type", payment_method_type, nullable=True
        )
        self.proportional_refund = proportional_refund
        self.reason = self._define_str("reason", reason, nullable=True)
        self.receipt_number = self._define_number(
            "receipt_number", receipt_number, nullable=True
        )
        self.status = self._define_str("status", status, nullable=True)
        self.updated_at = self._define_number("updated_at", updated_at, nullable=True)



from enum import Enum
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel


class ScreenColorDepth(Enum):
    """An enumeration representing different categories.

    :cvar _1: 1
    :vartype _1: int
    :cvar _4: 4
    :vartype _4: int
    :cvar _8: 8
    :vartype _8: int
    :cvar _15: 15
    :vartype _15: int
    :cvar _16: 16
    :vartype _16: int
    :cvar _24: 24
    :vartype _24: int
    :cvar _32: 32
    :vartype _32: int
    :cvar _48: 48
    :vartype _48: int
    """

    _1 = 1
    _4 = 4
    _8 = 8
    _15 = 15
    _16 = 16
    _24 = 24
    _32 = 32
    _48 = 48

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(map(lambda x: x.value, ScreenColorDepth._member_map_.values()))


@JsonMap({})
class ClientDetailsObject(BaseModel):
    """Describes the fields in the client_details object in REST messages for payments. The client_details object describes the browser that the customer is using. The client collects this information and sends it as part of the Create Payment request. This information is used for processing the 3DS version 2 authentication of the customer. Note that Client Details information is not returned in the API response and it does not appear in any webhooks.

    :param ip_address: IP address of the customer., defaults to None
    :type ip_address: str, optional
    :param java_enabled: Indicates whether the browser is enabled for Java., defaults to None
    :type java_enabled: bool, optional
    :param java_script_enabled: Indicates whether the browser is enabled for JavaScript., defaults to None
    :type java_script_enabled: bool, optional
    :param language: The language the browser is configured for, as defined in IETF BCP 47., defaults to None
    :type language: str, optional
    :param screen_color_depth: Indicates the screen color depth of the customer's browser, in bits., defaults to None
    :type screen_color_depth: ScreenColorDepth, optional
    :param screen_height: Height of the customer's screen, in pixels. 1-6 digits., defaults to None
    :type screen_height: int, optional
    :param screen_width: Width of the customer's screen, in pixels. 1-6 digits., defaults to None
    :type screen_width: int, optional
    :param time_zone_offset: Difference in minutes between UTC and the customer's time zone. Positive or negative integer., defaults to None
    :type time_zone_offset: int, optional
    """

    def __init__(
        self,
        ip_address: str = None,
        java_enabled: bool = None,
        java_script_enabled: bool = None,
        language: str = None,
        screen_color_depth: ScreenColorDepth = None,
        screen_height: int = None,
        screen_width: int = None,
        time_zone_offset: int = None,
    ):
        """Describes the fields in the client_details object in REST messages for payments. The client_details object describes the browser that the customer is using. The client collects this information and sends it as part of the Create Payment request. This information is used for processing the 3DS version 2 authentication of the customer. Note that Client Details information is not returned in the API response and it does not appear in any webhooks.

        :param ip_address: IP address of the customer., defaults to None
        :type ip_address: str, optional
        :param java_enabled: Indicates whether the browser is enabled for Java., defaults to None
        :type java_enabled: bool, optional
        :param java_script_enabled: Indicates whether the browser is enabled for JavaScript., defaults to None
        :type java_script_enabled: bool, optional
        :param language: The language the browser is configured for, as defined in IETF BCP 47., defaults to None
        :type language: str, optional
        :param screen_color_depth: Indicates the screen color depth of the customer's browser, in bits., defaults to None
        :type screen_color_depth: ScreenColorDepth, optional
        :param screen_height: Height of the customer's screen, in pixels. 1-6 digits., defaults to None
        :type screen_height: int, optional
        :param screen_width: Width of the customer's screen, in pixels. 1-6 digits., defaults to None
        :type screen_width: int, optional
        :param time_zone_offset: Difference in minutes between UTC and the customer's time zone. Positive or negative integer., defaults to None
        :type time_zone_offset: int, optional
        """
        self.ip_address = self._define_str("ip_address", ip_address, nullable=True)
        self.java_enabled = java_enabled
        self.java_script_enabled = java_script_enabled
        self.language = self._define_str("language", language, nullable=True)
        self.screen_color_depth = (
            self._enum_matching(
                screen_color_depth, ScreenColorDepth.list(), "screen_color_depth"
            )
            if screen_color_depth
            else None
        )
        self.screen_height = self._define_number(
            "screen_height", screen_height, nullable=True
        )
        self.screen_width = self._define_number(
            "screen_width", screen_width, nullable=True
        )
        self.time_zone_offset = self._define_number(
            "time_zone_offset", time_zone_offset, nullable=True
        )



from .utils.json_map import JsonMap
from .utils.base_model import BaseModel


@JsonMap({})
class PayoutEwallets(BaseModel):
    """PayoutEwallets

    :param amount: The amount of the payment to the wallet, in units of the currency defined in currency. Decimal. If the total amount of the payment is not fully accounted for, the balance appears in the client wallet. Relevant when percentage is not set., defaults to None
    :type amount: float, optional
    :param ewallet_id: ID of the wallet. String starting with **ewallet_**. Read-only., defaults to None
    :type ewallet_id: str, optional
    :param percent: The percentage that is paid to the wallet out of the total payment. Read-only., defaults to None
    :type percent: float, optional
    """

    def __init__(
        self, amount: float = None, ewallet_id: str = None, percent: float = None
    ):
        """PayoutEwallets

        :param amount: The amount of the payment to the wallet, in units of the currency defined in currency. Decimal. If the total amount of the payment is not fully accounted for, the balance appears in the client wallet. Relevant when percentage is not set., defaults to None
        :type amount: float, optional
        :param ewallet_id: ID of the wallet. String starting with **ewallet_**. Read-only., defaults to None
        :type ewallet_id: str, optional
        :param percent: The percentage that is paid to the wallet out of the total payment. Read-only., defaults to None
        :type percent: float, optional
        """
        self.amount = self._define_number("amount", amount, nullable=True)
        self.ewallet_id = self._define_str("ewallet_id", ewallet_id, nullable=True)
        self.percent = self._define_number("percent", percent, nullable=True)



from enum import Enum
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel


class EwalletTransactionBalanceType(Enum):
    """An enumeration representing different categories.

    :cvar AVAILABLEBALANCE: "available_balance"
    :vartype AVAILABLEBALANCE: str
    :cvar ONHOLDBALANCE: "on_hold_balance"
    :vartype ONHOLDBALANCE: str
    :cvar RECEIVEDBALANCE: "received_balance"
    :vartype RECEIVEDBALANCE: str
    :cvar RESERVEBALANCE: "reserve_balance"
    :vartype RESERVEBALANCE: str
    """

    AVAILABLEBALANCE = "available_balance"
    ONHOLDBALANCE = "on_hold_balance"
    RECEIVEDBALANCE = "received_balance"
    RESERVEBALANCE = "reserve_balance"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(lambda x: x.value, EwalletTransactionBalanceType._member_map_.values())
        )


class EwalletTransactionDestinationBalanceType(Enum):
    """An enumeration representing different categories.

    :cvar AVAILABLEBALANCE: "available_balance"
    :vartype AVAILABLEBALANCE: str
    :cvar ONHOLDBALANCE: "on_hold_balance"
    :vartype ONHOLDBALANCE: str
    :cvar RECEIVEDBALANCE: "received_balance"
    :vartype RECEIVEDBALANCE: str
    :cvar RESERVEBALANCE: "reserve_balance"
    :vartype RESERVEBALANCE: str
    """

    AVAILABLEBALANCE = "available_balance"
    ONHOLDBALANCE = "on_hold_balance"
    RECEIVEDBALANCE = "received_balance"
    RESERVEBALANCE = "reserve_balance"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(
                lambda x: x.value,
                EwalletTransactionDestinationBalanceType._member_map_.values(),
            )
        )


class EwalletTransactionSourceBalanceType(Enum):
    """An enumeration representing different categories.

    :cvar AVAILABLEBALANCE: "available_balance"
    :vartype AVAILABLEBALANCE: str
    :cvar ONHOLDBALANCE: "on_hold_balance"
    :vartype ONHOLDBALANCE: str
    :cvar RECEIVEDBALANCE: "received_balance"
    :vartype RECEIVEDBALANCE: str
    :cvar RESERVEBALANCE: "reserve_balance"
    :vartype RESERVEBALANCE: str
    """

    AVAILABLEBALANCE = "available_balance"
    ONHOLDBALANCE = "on_hold_balance"
    RECEIVEDBALANCE = "received_balance"
    RESERVEBALANCE = "reserve_balance"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(
                lambda x: x.value,
                EwalletTransactionSourceBalanceType._member_map_.values(),
            )
        )


@JsonMap({"id_": "id", "type_": "type"})
class EwalletTransaction(BaseModel):
    """EwalletTransaction

    :param metadata: metadata, defaults to None
    :type metadata: dict, optional
    :param amount: Amount of the transaction, in units of the currency defined in currency. Decimal, including the correct number of decimal places for the currency exponent, as defined in ISO 4217:2015., defaults to None
    :type amount: float, optional
    :param balance: The updated wallet balance after successful completion of the transaction., defaults to None
    :type balance: float, optional
    :param balance_type: Balance type affected by the transaction. One of the following, defaults to None
    :type balance_type: EwalletTransactionBalanceType, optional
    :param created_at: Time the transaction was made, in Unix time., defaults to None
    :type created_at: float, optional
    :param currency: Three-letter ISO 4217 code for the currency used in the amount field., defaults to None
    :type currency: str, optional
    :param destination_balance_type: The destination balance type when funds are transferred from one balance to another within the wallet., defaults to None
    :type destination_balance_type: EwalletTransactionDestinationBalanceType, optional
    :param destination_ewallet_id: ID of the wallet receiving the money. String starting with **ewallet_**. Response only., defaults to None
    :type destination_ewallet_id: str, optional
    :param ewallet_id: ID of the wallet. String starting with **ewallet_**., defaults to None
    :type ewallet_id: str, optional
    :param id_: ID of the transaction. String starting with wt_ or UUID., defaults to None
    :type id_: str, optional
    :param reason: Unique identifier of the wallet object. String starting with **ewallet_**., defaults to None
    :type reason: str, optional
    :param source_balance_type: The source balance type when funds are transferred from one balance to another within the wallet, defaults to None
    :type source_balance_type: EwalletTransactionSourceBalanceType, optional
    :param source_ewallet_id: ID of the wallet sending the money. String starting with **ewallet_**., defaults to None
    :type source_ewallet_id: str, optional
    :param status: Status of the transaction., defaults to None
    :type status: str, optional
    :param type_: Type of transaction, defaults to None
    :type type_: str, optional
    :param subtype: Subtype of the transaction., defaults to None
    :type subtype: float, optional
    :param action_data: action_data, defaults to None
    :type action_data: dict, optional
    """

    def __init__(
        self,
        metadata: dict = None,
        amount: float = None,
        balance: float = None,
        balance_type: EwalletTransactionBalanceType = None,
        created_at: float = None,
        currency: str = None,
        destination_balance_type: EwalletTransactionDestinationBalanceType = None,
        destination_ewallet_id: str = None,
        ewallet_id: str = None,
        id_: str = None,
        reason: str = None,
        source_balance_type: EwalletTransactionSourceBalanceType = None,
        source_ewallet_id: str = None,
        status: str = None,
        type_: str = None,
        subtype: float = None,
        action_data: dict = None,
    ):
        """EwalletTransaction

        :param metadata: metadata, defaults to None
        :type metadata: dict, optional
        :param amount: Amount of the transaction, in units of the currency defined in currency. Decimal, including the correct number of decimal places for the currency exponent, as defined in ISO 4217:2015., defaults to None
        :type amount: float, optional
        :param balance: The updated wallet balance after successful completion of the transaction., defaults to None
        :type balance: float, optional
        :param balance_type: Balance type affected by the transaction. One of the following, defaults to None
        :type balance_type: EwalletTransactionBalanceType, optional
        :param created_at: Time the transaction was made, in Unix time., defaults to None
        :type created_at: float, optional
        :param currency: Three-letter ISO 4217 code for the currency used in the amount field., defaults to None
        :type currency: str, optional
        :param destination_balance_type: The destination balance type when funds are transferred from one balance to another within the wallet., defaults to None
        :type destination_balance_type: EwalletTransactionDestinationBalanceType, optional
        :param destination_ewallet_id: ID of the wallet receiving the money. String starting with **ewallet_**. Response only., defaults to None
        :type destination_ewallet_id: str, optional
        :param ewallet_id: ID of the wallet. String starting with **ewallet_**., defaults to None
        :type ewallet_id: str, optional
        :param id_: ID of the transaction. String starting with wt_ or UUID., defaults to None
        :type id_: str, optional
        :param reason: Unique identifier of the wallet object. String starting with **ewallet_**., defaults to None
        :type reason: str, optional
        :param source_balance_type: The source balance type when funds are transferred from one balance to another within the wallet, defaults to None
        :type source_balance_type: EwalletTransactionSourceBalanceType, optional
        :param source_ewallet_id: ID of the wallet sending the money. String starting with **ewallet_**., defaults to None
        :type source_ewallet_id: str, optional
        :param status: Status of the transaction., defaults to None
        :type status: str, optional
        :param type_: Type of transaction, defaults to None
        :type type_: str, optional
        :param subtype: Subtype of the transaction., defaults to None
        :type subtype: float, optional
        :param action_data: action_data, defaults to None
        :type action_data: dict, optional
        """
        self.metadata = metadata
        self.amount = self._define_number("amount", amount, nullable=True)
        self.balance = self._define_number("balance", balance, nullable=True)
        self.balance_type = (
            self._enum_matching(
                balance_type, EwalletTransactionBalanceType.list(), "balance_type"
            )
            if balance_type
            else None
        )
        self.created_at = self._define_number("created_at", created_at, nullable=True)
        self.currency = self._define_str("currency", currency, nullable=True)
        self.destination_balance_type = (
            self._enum_matching(
                destination_balance_type,
                EwalletTransactionDestinationBalanceType.list(),
                "destination_balance_type",
            )
            if destination_balance_type
            else None
        )
        self.destination_ewallet_id = self._define_str(
            "destination_ewallet_id", destination_ewallet_id, nullable=True
        )
        self.ewallet_id = self._define_str("ewallet_id", ewallet_id, nullable=True)
        self.id_ = self._define_str("id_", id_, nullable=True)
        self.reason = self._define_str("reason", reason, nullable=True)
        self.source_balance_type = (
            self._enum_matching(
                source_balance_type,
                EwalletTransactionSourceBalanceType.list(),
                "source_balance_type",
            )
            if source_balance_type
            else None
        )
        self.source_ewallet_id = self._define_str(
            "source_ewallet_id", source_ewallet_id, nullable=True
        )
        self.status = self._define_str("status", status, nullable=True)
        self.type_ = self._define_str("type_", type_, nullable=True)
        self.subtype = self._define_number("subtype", subtype, nullable=True)
        self.action_data = action_data



from .utils.json_map import JsonMap
from .utils.base_model import BaseModel


@JsonMap({"id_": "id"})
class InlineResponse200_30DataTransactions(BaseModel):
    """InlineResponse200_30DataTransactions

    :param id_: ID of the issued card transaction object. String starting with **cit_**., defaults to None
    :type id_: str, optional
    :param ewallet: ID of the wallet the bank account number is issued to. String starting with **ewallet_**., defaults to None
    :type ewallet: str, optional
    :param account_id: ID of the virtual bank account., defaults to None
    :type account_id: str, optional
    :param account_id_type: Type of the virtual account number., defaults to None
    :type account_id_type: str, optional
    :param amount: Amount of the transaction., defaults to None
    :type amount: float, optional
    :param currency: Currency of the transaction. Three-letter ISO 4217., defaults to None
    :type currency: str, optional
    :param created_at: Time of creation of the transaction, in Unix time. Response only., defaults to None
    :type created_at: float, optional
    :param receiving_currency: receiving_currency, defaults to None
    :type receiving_currency: bool, optional
    """

    def __init__(
        self,
        id_: str = None,
        ewallet: str = None,
        account_id: str = None,
        account_id_type: str = None,
        amount: float = None,
        currency: str = None,
        created_at: float = None,
        receiving_currency: bool = None,
    ):
        """InlineResponse200_30DataTransactions

        :param id_: ID of the issued card transaction object. String starting with **cit_**., defaults to None
        :type id_: str, optional
        :param ewallet: ID of the wallet the bank account number is issued to. String starting with **ewallet_**., defaults to None
        :type ewallet: str, optional
        :param account_id: ID of the virtual bank account., defaults to None
        :type account_id: str, optional
        :param account_id_type: Type of the virtual account number., defaults to None
        :type account_id_type: str, optional
        :param amount: Amount of the transaction., defaults to None
        :type amount: float, optional
        :param currency: Currency of the transaction. Three-letter ISO 4217., defaults to None
        :type currency: str, optional
        :param created_at: Time of creation of the transaction, in Unix time. Response only., defaults to None
        :type created_at: float, optional
        :param receiving_currency: receiving_currency, defaults to None
        :type receiving_currency: bool, optional
        """
        self.id_ = self._define_str("id_", id_, nullable=True)
        self.ewallet = self._define_str("ewallet", ewallet, nullable=True)
        self.account_id = self._define_str("account_id", account_id, nullable=True)
        self.account_id_type = self._define_str(
            "account_id_type", account_id_type, nullable=True
        )
        self.amount = self._define_number("amount", amount, nullable=True)
        self.currency = self._define_str("currency", currency, nullable=True)
        self.created_at = self._define_number("created_at", created_at, nullable=True)
        self.receiving_currency = receiving_currency



from .utils.validator import Validator
from .utils.base_service import BaseService
from ..net.transport.serializer import Serializer
from ..models.utils.cast_models import cast_models
from ..models import (
    CardsAdjustmentBody,
    CardsAuthorizationBody,
    CardsClearingBody,
    CardsRefundBody,
    CardsReversalBody,
    CardsSimulateBlockBody,
    InlineResponse200_86,
    InlineResponse200_87,
    InlineResponse200_88,
    InlineResponse200_89,
)


class IssuingSimulationsService(BaseService):

    @cast_models
    def simulate_block_card(
        self,
        request_body: CardsSimulateBlockBody,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        idempotency: str = None,
    ) -> InlineResponse200_86:
        """Simulate a third-party stop notice for an issued card.<BR>Relevant to the sandbox.<BR>Rapyd sets the card status to **BLO** (blocked) and sends you the Card Issuing Blocked Webhook.<BR> To unblock the card, see Update Card Status.<BR>**Prerequisites**:<BR> * Issue Card <BR> * Activate Issued Card Using API

        :param request_body: The request body.
        :type request_body: CardsSimulateBlockBody
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Returns card details.
        :rtype: InlineResponse200_86
        """

        Validator(CardsSimulateBlockBody).validate(request_body)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/issuing/cards/simulate_block",
                self.get_default_headers(),
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_86._unmap(response)

    @cast_models
    def simulate_card_transaction_authorization_request_eea(
        self,
        request_body: CardsAuthorizationBody,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        idempotency: str = None,
    ) -> InlineResponse200_87:
        """Simulate Rapyd’s request to your authorization server.<BR> If you have implemented Remote Authorization, Rapyd sends authorization requests to your remote authorization server when the card network requests approval of a transaction on a card issued to a Rapyd Wallet. Use this method in the sandbox.<BR>The response contains all the fields you must send to Rapyd when you receive an authorization request in production. See also Remote Authorization.<BR> When `financial_impact` is set to **credit**, this method triggers 'Card Issuing Credit Webhook'.   When `financial_impact` is set to debit, this method triggers 'Card Issuing Authorization Approved' Webhook. Funds in the wallet move from **available balance** to **on-hold balance**. <BR> To simulate deducting the funds from the wallet, run 'Simulate Clearing a Card Transaction - EEA'.<BR> This method applies to cards issued in the European Economic Area (EEA). For cards issued outside the EEA, see Simulate a Card Transaction Authorization Request - Non-EEA.<BR>**Prerequisites**<BR> * A **company** wallet with a **personal** contact. Run 'Create Wallet' and then 'Add Contact to Wallet'<BR>* Issue Card<BR> * Activate Issued Card Using API<BR> * Add Funds to Wallet Account

        :param request_body: The request body.
        :type request_body: CardsAuthorizationBody
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Returns card and transaction details.
        :rtype: InlineResponse200_87
        """

        Validator(CardsAuthorizationBody).validate(request_body)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/issuing/cards/authorization",
                self.get_default_headers(),
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_87._unmap(response)

    @cast_models
    def simulate_card_transaction_authorization_reversal_eea(
        self,
        request_body: CardsReversalBody,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        idempotency: str = None,
    ) -> InlineResponse200_88:
        """Simulate a retail location’s reversal of a card authorization.<BR>The retail location where a card was used for a purchase can send a reversal through the card network to cancel a previous authorization request. This can occur when the transaction is not completed within the time required by the card issuer. When Rapyd receives a request for reversal from the card network, it reverses the transaction and sends you a webhook. Use this method in the sandbox.<BR>This method triggers Card Issuing Reversal Webhook . This method moves the reversed funds in the wallet from **on-hold** balance to **available balance**. <BR>This method applies to cards issued in the European Economic Area (EEA). For cards issued outside the EEA, see 'Simulate a Card Transaction Authorization Reversal - Non-EEA' in Rapyd's online API reference.<BR>**Prerequisites:**<BR> * Issue Card<BR> * Activate Issued Card Using API<BR> * Simulate a Card Transaction Authorization Request - EEA

        :param request_body: The request body.
        :type request_body: CardsReversalBody
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Returns card and transaction details.
        :rtype: InlineResponse200_88
        """

        Validator(CardsReversalBody).validate(request_body)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/issuing/cards/reversal", self.get_default_headers()
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_88._unmap(response)

    @cast_models
    def simulate_clearing_card_transaction_eea(
        self,
        request_body: CardsClearingBody,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        idempotency: str = None,
    ) -> InlineResponse200_89:
        """Simulate clearing a debit transaction.<BR>This method triggers Webhook - Card Issuing Sale. When a fee is charged for an ATM transaction, this method also triggers Webhook - Card Issuing ATM Fee. Use this method in the sandbox.<BR>This method deducts funds from the **on-hold** balance in the wallet.<BR>**Prerequisites:**<BR> * Issue Card<BR> * Activate Issued Card Using API<BR> * Simulate a Card Transaction Authorization Request - EEA

        :param request_body: The request body.
        :type request_body: CardsClearingBody
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Returns card clearing and transaction details.
        :rtype: InlineResponse200_89
        """

        Validator(CardsClearingBody).validate(request_body)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/issuing/cards/clearing", self.get_default_headers()
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_89._unmap(response)

    @cast_models
    def simulate_card_refund_eea(
        self,
        request_body: CardsRefundBody,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        idempotency: str = None,
    ) -> InlineResponse200_89:
        """Simulate a retail location making a refund to an issued card.<BR> This method triggers Card Issuing Refund Webhook. This method credits the **available balance** of the wallet with the refunded amount. See 'Wallet Balance Types' in online API reference. Use this method in the sandbox.<BR> This method applies to cards issued in the European Economic Area (EEA). For cards issued outside the EEA, see 'Simulate a Card Refund - Non-EEA'  in online API reference. <BR>**Prerequisites:**<BR>* An issued card. See 'Issue Card'.<BR>* Activate the card via API or by using a Hosted Page.<BR>* Authorization of a card transaction. See 'Simulate a Card Transaction Authorization Request - EEA'.

        :param request_body: The request body.
        :type request_body: CardsRefundBody
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Returns card refund and transaction details.
        :rtype: InlineResponse200_89
        """

        Validator(CardsRefundBody).validate(request_body)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/issuing/cards/refund", self.get_default_headers()
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_89._unmap(response)

    @cast_models
    def simulate_card_transaction_adjustment_eea(
        self,
        request_body: CardsAdjustmentBody,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        idempotency: str = None,
    ) -> InlineResponse200_89:
        """Use this method in the sandbox to simulate an adjustment to a card transaction. A retail location can adjust the amount of a transaction made by an issued card. The adjustment is either a credit or a debit. This method triggers Card Issuing Adjustment Webhook. This method updates the **available balance** of the wallet with the credit or deduction. See 'Wallet Balance Types' in online API Reference. This method applies to cards issued in the European Economic Area (EEA). <BR>**Prerequisites:**<BR>* Issue card<BR>* Activate the card via API <BR>* Simulate a Card Transaction Authorization Request - EEA

        :param request_body: The request body.
        :type request_body: CardsAdjustmentBody
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Returns card adjustment and transaction details.
        :rtype: InlineResponse200_89
        """

        Validator(CardsAdjustmentBody).validate(request_body)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/issuing/cards/adjustment",
                self.get_default_headers(),
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_89._unmap(response)



from __future__ import annotations
from typing import Union
from typing import List
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.one_of_base_model import OneOfBaseModel
from .status_1 import Status1
from .card_issuing import CardIssuing
from .card_issuing_masked import CardIssuingMasked


class InlineResponse200_75DataGuard(OneOfBaseModel):
    class_list = {"CardIssuing": CardIssuing, "CardIssuingMasked": CardIssuingMasked}


InlineResponse200_75Data = Union[CardIssuing, CardIssuingMasked]


@JsonMap({})
class InlineResponse200_75(BaseModel):
    """InlineResponse200_75

    :param status: status, defaults to None
    :type status: Status1, optional
    :param data: data, defaults to None
    :type data: List[InlineResponse200_75Data], optional
    """

    def __init__(
        self, status: Status1 = None, data: List[InlineResponse200_75Data] = None
    ):
        """InlineResponse200_75

        :param status: status, defaults to None
        :type status: Status1, optional
        :param data: data, defaults to None
        :type data: List[InlineResponse200_75Data], optional
        """
        self.status = self._define_object(status, Status1)
        self.data = self._define_list(data, InlineResponse200_75Data)



from .utils.json_map import JsonMap
from .utils.base_model import BaseModel


@JsonMap({})
class CardsClearingBody(BaseModel):
    """CardsClearingBody

    :param amount: The amount of the authorization, in units of the `currency` defined in currency. Decimal.
    :type amount: float
    :param authorization_id: ID of the authorization. String starting with **cardauth_**. Use the value of `id` in the response to Simulate a Card Transaction Authorization Request - EEA.
    :type authorization_id: str
    :param card_id: ID of the card. String starting with **card_**.
    :type card_id: str
    :param category: Type of charge: **ATM**. Required when `fee_amount` is set., defaults to None
    :type category: str, optional
    :param currency: Defines the currency for the transaction. Three-letter ISO 4217 code.
    :type currency: str
    :param fee_amount: The amount of the fee charged for the transaction, in units of the currency defined in `currency`. Decimal. Required when `category` is set., defaults to None
    :type fee_amount: str, optional
    """

    def __init__(
        self,
        amount: float,
        authorization_id: str,
        card_id: str,
        currency: str,
        category: str = None,
        fee_amount: str = None,
    ):
        """CardsClearingBody

        :param amount: The amount of the authorization, in units of the `currency` defined in currency. Decimal.
        :type amount: float
        :param authorization_id: ID of the authorization. String starting with **cardauth_**. Use the value of `id` in the response to Simulate a Card Transaction Authorization Request - EEA.
        :type authorization_id: str
        :param card_id: ID of the card. String starting with **card_**.
        :type card_id: str
        :param category: Type of charge: **ATM**. Required when `fee_amount` is set., defaults to None
        :type category: str, optional
        :param currency: Defines the currency for the transaction. Three-letter ISO 4217 code.
        :type currency: str
        :param fee_amount: The amount of the fee charged for the transaction, in units of the currency defined in `currency`. Decimal. Required when `category` is set., defaults to None
        :type fee_amount: str, optional
        """
        self.amount = amount
        self.authorization_id = authorization_id
        self.card_id = card_id
        self.category = self._define_str("category", category, nullable=True)
        self.currency = currency
        self.fee_amount = self._define_str("fee_amount", fee_amount, nullable=True)



from .utils.json_map import JsonMap
from .utils.base_model import BaseModel


@JsonMap({"id_": "id"})
class Address1(BaseModel):
    """address associated with this specific Rapyd entity Payment/Customer etc.

    :param canton: Name of the canton administrative subdivision, as used in banking, defaults to None
    :type canton: str, optional
    :param city: City name portion of the address. Required for issuance of a card to an eWallet contact, defaults to None
    :type city: str, optional
    :param country: country, defaults to None
    :type country: str, optional
    :param created_at: Time of creation of the payment, in Unix time. Response only., defaults to None
    :type created_at: float, optional
    :param district: Name of the district administrative subdivision, as used in banking, defaults to None
    :type district: str, optional
    :param id_: ID of the Address object. String starting with address_, defaults to None
    :type id_: str, optional
    :param line_1: Line 1 of the address, such as a building number and street name, defaults to None
    :type line_1: str, optional
    :param line_2: Line 2 of the address, such as a suite or apartment number, or the name of a named building, defaults to None
    :type line_2: str, optional
    :param line_3: Line 3 of the address, defaults to None
    :type line_3: str, optional
    :param metadata: A JSON object defined by the client, defaults to None
    :type metadata: dict, optional
    :param name: The name of a contact person or an `in care of` person at this address, defaults to None
    :type name: str, optional
    :param phone_number: Phone number associated with this specific address in E.164 format. Must be unique, defaults to None
    :type phone_number: str, optional
    :param state: state, defaults to None
    :type state: str, optional
    :param zip: Postal code portion of the address, defaults to None
    :type zip: str, optional
    """

    def __init__(
        self,
        canton: str = None,
        city: str = None,
        country: str = None,
        created_at: float = None,
        district: str = None,
        id_: str = None,
        line_1: str = None,
        line_2: str = None,
        line_3: str = None,
        metadata: dict = None,
        name: str = None,
        phone_number: str = None,
        state: str = None,
        zip: str = None,
    ):
        """address associated with this specific Rapyd entity Payment/Customer etc.

        :param canton: Name of the canton administrative subdivision, as used in banking, defaults to None
        :type canton: str, optional
        :param city: City name portion of the address. Required for issuance of a card to an eWallet contact, defaults to None
        :type city: str, optional
        :param country: country, defaults to None
        :type country: str, optional
        :param created_at: Time of creation of the payment, in Unix time. Response only., defaults to None
        :type created_at: float, optional
        :param district: Name of the district administrative subdivision, as used in banking, defaults to None
        :type district: str, optional
        :param id_: ID of the Address object. String starting with address_, defaults to None
        :type id_: str, optional
        :param line_1: Line 1 of the address, such as a building number and street name, defaults to None
        :type line_1: str, optional
        :param line_2: Line 2 of the address, such as a suite or apartment number, or the name of a named building, defaults to None
        :type line_2: str, optional
        :param line_3: Line 3 of the address, defaults to None
        :type line_3: str, optional
        :param metadata: A JSON object defined by the client, defaults to None
        :type metadata: dict, optional
        :param name: The name of a contact person or an `in care of` person at this address, defaults to None
        :type name: str, optional
        :param phone_number: Phone number associated with this specific address in E.164 format. Must be unique, defaults to None
        :type phone_number: str, optional
        :param state: state, defaults to None
        :type state: str, optional
        :param zip: Postal code portion of the address, defaults to None
        :type zip: str, optional
        """
        self.canton = self._define_str("canton", canton, nullable=True)
        self.city = self._define_str("city", city, nullable=True)
        self.country = self._define_str(
            "country",
            country,
            nullable=True,
            pattern="Name of the country. Two-letter ISO 3166-1 alpha-2 code.",
        )
        self.created_at = self._define_number("created_at", created_at, nullable=True)
        self.district = self._define_str("district", district, nullable=True)
        self.id_ = self._define_str("id_", id_, nullable=True)
        self.line_1 = self._define_str("line_1", line_1, nullable=True)
        self.line_2 = self._define_str("line_2", line_2, nullable=True)
        self.line_3 = self._define_str("line_3", line_3, nullable=True)
        self.metadata = metadata
        self.name = self._define_str("name", name, nullable=True)
        self.phone_number = self._define_str(
            "phone_number", phone_number, nullable=True
        )
        self.state = self._define_str("state", state, nullable=True)
        self.zip = self._define_str("zip", zip, nullable=True)



from __future__ import annotations
from enum import Enum
from typing import Union
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.one_of_base_model import OneOfBaseModel
from .discount import Discount
from .subscription_items import SubscriptionItems


class SubscriptionBillingCycleAnchorGuard(OneOfBaseModel):
    class_list = {"str": str, "float": float}


SubscriptionBillingCycleAnchor = Union[str, float]


class SubscriptionStatus(Enum):
    """An enumeration representing different categories.

    :cvar ACTIVE: "active"
    :vartype ACTIVE: str
    :cvar CANCELED: "canceled"
    :vartype CANCELED: str
    :cvar PASTDUE: "past_due"
    :vartype PASTDUE: str
    :cvar TRIALING: "trialing"
    :vartype TRIALING: str
    :cvar UNPAID: "unpaid"
    :vartype UNPAID: str
    """

    ACTIVE = "active"
    CANCELED = "canceled"
    PASTDUE = "past_due"
    TRIALING = "trialing"
    UNPAID = "unpaid"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(map(lambda x: x.value, SubscriptionStatus._member_map_.values()))


class SubscriptionType(Enum):
    """An enumeration representing different categories.

    :cvar PAYOUT: "payout"
    :vartype PAYOUT: str
    :cvar PAYMENT: "payment"
    :vartype PAYMENT: str
    """

    PAYOUT = "payout"
    PAYMENT = "payment"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(map(lambda x: x.value, SubscriptionType._member_map_.values()))


@JsonMap({"id_": "id", "type_": "type"})
class Subscription(BaseModel):
    """Subscription

    :param billing: Determines the method of billing at the end of the billing cycle. Set to pay_automatically - Rapyd generates a 'payment' object, then attempts to pay it using the designated payment method., defaults to None
    :type billing: str, optional
    :param billing_cycle_anchor: Determines the start of the next full billing cycle, as defined in the plan described in the 'items'. One of the following values:  * now - The present day. The next billing cycle starts right now.  * Timestamp in Unix time - A time in the future, at or after the end of the free trial period, not more than the length of the billing cycle. The current billing cycle will be shorter than all other billing cycles. Relevant to creation of the subscription.  * unchanged - The original billing cycle anchor is unchanged. Relevant to updating a subscription. , defaults to None
    :type billing_cycle_anchor: SubscriptionBillingCycleAnchor, optional
    :param cancel_at_period_end: Determines the last date that charges accrue. true - Charges accrue until the end of the current billing period, then the subscription is canceled.  * When no trial period is set, after cancel_at_period_end is set to true the subscription will not be renewed at the next interval.  * When a trial period is set after cancel_at_period_end is set to true, the subscription will not begin. false - This is the default.  * When the subscription is created, no end is defined. * When Cancel Subscription is run, charges stop immediately and the subscription is canceled. , defaults to None
    :type cancel_at_period_end: bool, optional
    :param canceled_at: Date and time that the subscription is canceled by the customer or by the client. Unix time. Response only., defaults to None
    :type canceled_at: float, optional
    :param created_at: Time of creation of this subscription, in Unix time. Response only., defaults to None
    :type created_at: float, optional
    :param current_period_end: End of the current billing cycle, in Unix time. Response only., defaults to None
    :type current_period_end: float, optional
    :param current_period_start: Start of the current billing cycle, in Unix time. Response only., defaults to None
    :type current_period_start: float, optional
    :param customer_token: ID of the customer who pays for this subscription. String starting with cus_. Response only., defaults to None
    :type customer_token: str, optional
    :param days_until_due: Number of days from the invoice date for customer to complete the payment., defaults to None
    :type days_until_due: float, optional
    :param discount: Describes the fields relating to discounts in REST messages and webhooks for customer profiles and subscriptions Contains information about the coupon that applies to the customer. Read-only field. Adding a discount is a 2-step process: \<BR\> 1. Create Coupon, which returns a coupon ID. \<BR\>2. Add the coupon ID to the coupon field of the customer with Create Customer or Update Customer., defaults to None
    :type discount: Discount, optional
    :param ended_at: Date that the subscription was canceled or otherwise terminated. Unix time. Response only., defaults to None
    :type ended_at: float, optional
    :param id_: ID of the Subscription object. String starting with sub_., defaults to None
    :type id_: str, optional
    :param metadata: A JSON object defined by the client., defaults to None
    :type metadata: dict, optional
    :param payment_fields: Additional payment_options fields., defaults to None
    :type payment_fields: dict, optional
    :param payment_method: Payment method ID or Payment Method object. if not specified in this field, the payment method is the default payment method specified for the customer., defaults to None
    :type payment_method: str, optional
    :param payout_fields: Reserved., defaults to None
    :type payout_fields: dict, optional
    :param simultaneous_invoice: Determines whether each upcoming subscription invoice is issued immediately.  * true - The invoice is issued when the subscription is created.  * false - The invoice is issued an hour after the subscription is created. This is the default. , defaults to None
    :type simultaneous_invoice: bool, optional
    :param status: Status of the subscription. One of the following:  * active - The customer is currently paying for this subscription.  * canceled - The customer has canceled this subscription, but it remains in the Rapyd database.  * past_due - Payment for this subscription was not received by the end of the billing period.  * trialing - The subscription is in its free trial period. * unpaid - An error occurred in the payment for this subscription. Response only. , defaults to None
    :type status: SubscriptionStatus, optional
    :param subscription_items: subscription_items, defaults to None
    :type subscription_items: SubscriptionItems, optional
    :param tax_percent: The percentage tax rate that is applied to the subtotal of the invoice, after subtracting all discounts. Decimal. Range: 0-100, with up to four decimal places., defaults to None
    :type tax_percent: float, optional
    :param trial_end: The date and time of the end of the customer's free trial period, in Unix time, or the string now. Takes precedence over trial_period_days. If trial_end is not set by the client, Rapyd calculates this date by adding trial_period_days to the date in created_at. Relevant when trial_period_days is not defined., defaults to None
    :type trial_end: float, optional
    :param trial_start: Date and time of the start of the customer's free trial period, in Unix time. Response only., defaults to None
    :type trial_start: float, optional
    :param type_: The type of the subscription. One of the following values: * payment  * payout  Response only. , defaults to None
    :type type_: SubscriptionType, optional
    """

    def __init__(
        self,
        billing: str = None,
        billing_cycle_anchor: SubscriptionBillingCycleAnchor = None,
        cancel_at_period_end: bool = None,
        canceled_at: float = None,
        created_at: float = None,
        current_period_end: float = None,
        current_period_start: float = None,
        customer_token: str = None,
        days_until_due: float = None,
        discount: Discount = None,
        ended_at: float = None,
        id_: str = None,
        metadata: dict = None,
        payment_fields: dict = None,
        payment_method: str = None,
        payout_fields: dict = None,
        simultaneous_invoice: bool = None,
        status: SubscriptionStatus = None,
        subscription_items: SubscriptionItems = None,
        tax_percent: float = None,
        trial_end: float = None,
        trial_start: float = None,
        type_: SubscriptionType = None,
    ):
        """Subscription

        :param billing: Determines the method of billing at the end of the billing cycle. Set to pay_automatically - Rapyd generates a 'payment' object, then attempts to pay it using the designated payment method., defaults to None
        :type billing: str, optional
        :param billing_cycle_anchor: Determines the start of the next full billing cycle, as defined in the plan described in the 'items'. One of the following values:  * now - The present day. The next billing cycle starts right now.  * Timestamp in Unix time - A time in the future, at or after the end of the free trial period, not more than the length of the billing cycle. The current billing cycle will be shorter than all other billing cycles. Relevant to creation of the subscription.  * unchanged - The original billing cycle anchor is unchanged. Relevant to updating a subscription. , defaults to None
        :type billing_cycle_anchor: SubscriptionBillingCycleAnchor, optional
        :param cancel_at_period_end: Determines the last date that charges accrue. true - Charges accrue until the end of the current billing period, then the subscription is canceled.  * When no trial period is set, after cancel_at_period_end is set to true the subscription will not be renewed at the next interval.  * When a trial period is set after cancel_at_period_end is set to true, the subscription will not begin. false - This is the default.  * When the subscription is created, no end is defined. * When Cancel Subscription is run, charges stop immediately and the subscription is canceled. , defaults to None
        :type cancel_at_period_end: bool, optional
        :param canceled_at: Date and time that the subscription is canceled by the customer or by the client. Unix time. Response only., defaults to None
        :type canceled_at: float, optional
        :param created_at: Time of creation of this subscription, in Unix time. Response only., defaults to None
        :type created_at: float, optional
        :param current_period_end: End of the current billing cycle, in Unix time. Response only., defaults to None
        :type current_period_end: float, optional
        :param current_period_start: Start of the current billing cycle, in Unix time. Response only., defaults to None
        :type current_period_start: float, optional
        :param customer_token: ID of the customer who pays for this subscription. String starting with cus_. Response only., defaults to None
        :type customer_token: str, optional
        :param days_until_due: Number of days from the invoice date for customer to complete the payment., defaults to None
        :type days_until_due: float, optional
        :param discount: Describes the fields relating to discounts in REST messages and webhooks for customer profiles and subscriptions Contains information about the coupon that applies to the customer. Read-only field. Adding a discount is a 2-step process: \<BR\> 1. Create Coupon, which returns a coupon ID. \<BR\>2. Add the coupon ID to the coupon field of the customer with Create Customer or Update Customer., defaults to None
        :type discount: Discount, optional
        :param ended_at: Date that the subscription was canceled or otherwise terminated. Unix time. Response only., defaults to None
        :type ended_at: float, optional
        :param id_: ID of the Subscription object. String starting with sub_., defaults to None
        :type id_: str, optional
        :param metadata: A JSON object defined by the client., defaults to None
        :type metadata: dict, optional
        :param payment_fields: Additional payment_options fields., defaults to None
        :type payment_fields: dict, optional
        :param payment_method: Payment method ID or Payment Method object. if not specified in this field, the payment method is the default payment method specified for the customer., defaults to None
        :type payment_method: str, optional
        :param payout_fields: Reserved., defaults to None
        :type payout_fields: dict, optional
        :param simultaneous_invoice: Determines whether each upcoming subscription invoice is issued immediately.  * true - The invoice is issued when the subscription is created.  * false - The invoice is issued an hour after the subscription is created. This is the default. , defaults to None
        :type simultaneous_invoice: bool, optional
        :param status: Status of the subscription. One of the following:  * active - The customer is currently paying for this subscription.  * canceled - The customer has canceled this subscription, but it remains in the Rapyd database.  * past_due - Payment for this subscription was not received by the end of the billing period.  * trialing - The subscription is in its free trial period. * unpaid - An error occurred in the payment for this subscription. Response only. , defaults to None
        :type status: SubscriptionStatus, optional
        :param subscription_items: subscription_items, defaults to None
        :type subscription_items: SubscriptionItems, optional
        :param tax_percent: The percentage tax rate that is applied to the subtotal of the invoice, after subtracting all discounts. Decimal. Range: 0-100, with up to four decimal places., defaults to None
        :type tax_percent: float, optional
        :param trial_end: The date and time of the end of the customer's free trial period, in Unix time, or the string now. Takes precedence over trial_period_days. If trial_end is not set by the client, Rapyd calculates this date by adding trial_period_days to the date in created_at. Relevant when trial_period_days is not defined., defaults to None
        :type trial_end: float, optional
        :param trial_start: Date and time of the start of the customer's free trial period, in Unix time. Response only., defaults to None
        :type trial_start: float, optional
        :param type_: The type of the subscription. One of the following values: * payment  * payout  Response only. , defaults to None
        :type type_: SubscriptionType, optional
        """
        self.billing = self._define_str("billing", billing, nullable=True)
        self.billing_cycle_anchor = SubscriptionBillingCycleAnchorGuard.return_one_of(
            billing_cycle_anchor
        )
        self.cancel_at_period_end = cancel_at_period_end
        self.canceled_at = self._define_number(
            "canceled_at", canceled_at, nullable=True
        )
        self.created_at = self._define_number("created_at", created_at, nullable=True)
        self.current_period_end = self._define_number(
            "current_period_end", current_period_end, nullable=True
        )
        self.current_period_start = self._define_number(
            "current_period_start", current_period_start, nullable=True
        )
        self.customer_token = self._define_str(
            "customer_token", customer_token, nullable=True
        )
        self.days_until_due = self._define_number(
            "days_until_due", days_until_due, nullable=True
        )
        self.discount = self._define_object(discount, Discount)
        self.ended_at = self._define_number("ended_at", ended_at, nullable=True)
        self.id_ = self._define_str("id_", id_, nullable=True)
        self.metadata = metadata
        self.payment_fields = payment_fields
        self.payment_method = self._define_str(
            "payment_method", payment_method, nullable=True
        )
        self.payout_fields = payout_fields
        self.simultaneous_invoice = simultaneous_invoice
        self.status = (
            self._enum_matching(status, SubscriptionStatus.list(), "status")
            if status
            else None
        )
        self.subscription_items = self._define_object(
            subscription_items, SubscriptionItems
        )
        self.tax_percent = self._define_number(
            "tax_percent", tax_percent, nullable=True
        )
        self.trial_end = self._define_number("trial_end", trial_end, nullable=True)
        self.trial_start = self._define_number(
            "trial_start", trial_start, nullable=True
        )
        self.type_ = (
            self._enum_matching(type_, SubscriptionType.list(), "type_")
            if type_
            else None
        )



from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .verify_hosted_app_response_merchant_details_merchant_customer_support import (
    VerifyHostedAppResponseMerchantDetailsMerchantCustomerSupport,
)


@JsonMap({})
class VerifyHostedAppResponseMerchantDetails(BaseModel):
    """Object containing information about the merchant.

    :param merchant_alias: The name that appears on the hosted page when merchant_logo is not specified. To change this value, contact Client Support. Response only., defaults to None
    :type merchant_alias: str, optional
    :param merchant_language: Determines the default language of the application page.  The values are documented in List Hosted Page Supported Languages. , defaults to None
    :type merchant_language: str, optional
    :param merchant_logo: URL for the image of the client's logo. Response only. To configure this feature, use the Client Portal., defaults to None
    :type merchant_logo: str, optional
    :param merchant_website: The URL where the customer is redirected after exiting the hosted page., defaults to None
    :type merchant_website: str, optional
    :param merchant_color: merchant_color, defaults to None
    :type merchant_color: str, optional
    :param merchant_design: merchant_design, defaults to None
    :type merchant_design: str, optional
    :param merchant_customer_support: merchant_customer_support, defaults to None
    :type merchant_customer_support: VerifyHostedAppResponseMerchantDetailsMerchantCustomerSupport, optional
    :param merchant_terms: merchant_terms, defaults to None
    :type merchant_terms: str, optional
    :param merchant_privacy_policy: merchant_privacy_policy, defaults to None
    :type merchant_privacy_policy: str, optional
    """

    def __init__(
        self,
        merchant_alias: str = None,
        merchant_language: str = None,
        merchant_logo: str = None,
        merchant_website: str = None,
        merchant_color: str = None,
        merchant_design: str = None,
        merchant_customer_support: VerifyHostedAppResponseMerchantDetailsMerchantCustomerSupport = None,
        merchant_terms: str = None,
        merchant_privacy_policy: str = None,
    ):
        """Object containing information about the merchant.

        :param merchant_alias: The name that appears on the hosted page when merchant_logo is not specified. To change this value, contact Client Support. Response only., defaults to None
        :type merchant_alias: str, optional
        :param merchant_language: Determines the default language of the application page.  The values are documented in List Hosted Page Supported Languages. , defaults to None
        :type merchant_language: str, optional
        :param merchant_logo: URL for the image of the client's logo. Response only. To configure this feature, use the Client Portal., defaults to None
        :type merchant_logo: str, optional
        :param merchant_website: The URL where the customer is redirected after exiting the hosted page., defaults to None
        :type merchant_website: str, optional
        :param merchant_color: merchant_color, defaults to None
        :type merchant_color: str, optional
        :param merchant_design: merchant_design, defaults to None
        :type merchant_design: str, optional
        :param merchant_customer_support: merchant_customer_support, defaults to None
        :type merchant_customer_support: VerifyHostedAppResponseMerchantDetailsMerchantCustomerSupport, optional
        :param merchant_terms: merchant_terms, defaults to None
        :type merchant_terms: str, optional
        :param merchant_privacy_policy: merchant_privacy_policy, defaults to None
        :type merchant_privacy_policy: str, optional
        """
        self.merchant_alias = self._define_str(
            "merchant_alias", merchant_alias, nullable=True
        )
        self.merchant_language = self._define_str(
            "merchant_language", merchant_language, nullable=True
        )
        self.merchant_logo = self._define_str(
            "merchant_logo", merchant_logo, nullable=True
        )
        self.merchant_website = self._define_str(
            "merchant_website", merchant_website, nullable=True
        )
        self.merchant_color = self._define_str(
            "merchant_color", merchant_color, nullable=True
        )
        self.merchant_design = self._define_str(
            "merchant_design", merchant_design, nullable=True
        )
        self.merchant_customer_support = self._define_object(
            merchant_customer_support,
            VerifyHostedAppResponseMerchantDetailsMerchantCustomerSupport,
        )
        self.merchant_terms = self._define_str(
            "merchant_terms", merchant_terms, nullable=True
        )
        self.merchant_privacy_policy = self._define_str(
            "merchant_privacy_policy", merchant_privacy_policy, nullable=True
        )
